@@ -8,7 +8,7 @@ package Net::Async::HTTP::Connection;
 use strict;
 use warnings;
 
-our $VERSION = '0.37';
+our $VERSION = '0.38';
 
 use Carp;
 
@@ -61,7 +61,7 @@ sub configure
       $params{on_closed} = sub {
          my $self = shift;
 
-         $self->debug_printf( "CLOSED" );
+         $self->debug_printf( "CLOSED in-flight=$self->{requests_in_flight}" );
 
          $self->error_all( "Connection closed" );
 
@@ -97,7 +97,6 @@ sub connect
       %args
    )->on_done( sub {
       $self->debug_printf( "CONNECTED" );
-      $self->ready;
    });
 }
 
@@ -229,7 +228,10 @@ sub request
    # TODO: Cancelling a request Future shouldn't necessarily close the socket
    # if we haven't even started writing the request yet. But we can't know
    # that currently.
-   $f->on_cancel( sub { $self->close_now } );
+   $f->on_cancel( sub {
+      $self->debug_printf( "CLOSE on_cancel" );
+      $self->close_now;
+   });
 
    my $stall_timer;
    if( $args{stall_timeout} ) {
@@ -436,7 +438,7 @@ sub _mk_on_read_header
             return undef;
          }
 
-         $on_body_chunk->( $final ) if length $final;
+         $on_body_chunk->( $final ) if defined $final and length $final;
 
          my $response = $on_body_chunk->();
          my $e = eval { $f->done( $response ) unless $f->is_cancelled; 1 } ? undef : $@;
